package nl.dragontamerfred.lagmon.commands;

import nl.dragontamerfred.lagmon.Config;
import nl.dragontamerfred.lagmon.commands.lmrestart.Help;
import nl.dragontamerfred.lagmon.commands.lmrestart.Now;
import nl.dragontamerfred.lagmon.commands.lmrestart.Vote;
import nl.dragontamerfred.lagmon.types.ICommand;
import nl.dragontamerfred.lagmon.types.ISubCommand;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

public class CommandLmrestart implements CommandExecutor, TabCompleter, ICommand {

    private static HashMap<String, ISubCommand> subCommands = new HashMap<>();

    public CommandLmrestart() {
        subCommands.put("help", new Help());
        subCommands.put("now", new Now());
        subCommands.put("vote", new Vote());
    }

    // {PLAYER} {VOTE_AMOUNT} {VOTE_PERCENTAGE} {TIME}
    @Override
    public boolean onCommand(CommandSender sender, Command command, String label, String[] args) {
        if (!sender.hasPermission("lagmon.restart.vote")) {
            sender.sendMessage(Config.RESTART_MESSAGE_NO_PERM);
            return true;
        }

        if (args.length <= 0)
            return subCommands.get("help").execute(this, sender, args);

        if (subCommands.containsKey(args[0])) {
            return subCommands.get(args[0]).execute(this, sender, args);
        } else {
            return subCommands.get("help").execute(this, sender, args);
        }
    }

    @Override
    public List<String> onTabComplete(CommandSender sender, Command command, String label, String[] args) {
        List<String> tabComplete = new ArrayList<>();
        for (String subCommand : subCommands.keySet()) {
            if (subCommand.startsWith(args[0]))
                tabComplete.add(subCommand);

        }
        return tabComplete;
    }

    @Override
    public HashMap<String, ISubCommand> getSubCommands() {
        return subCommands;
    }
}
