package nl.dragontamerfred.lagmon.components;

import nl.dragontamerfred.lagmon.Config;
import nl.dragontamerfred.lagmon.LagMon;
import nl.dragontamerfred.lagmon.Util;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;

import java.util.HashSet;


public class ComponentRestart {

    private static VoteStatus status = VoteStatus.STOPPED;
    private static BukkitTask voteTask;
    private static HashSet<String> votes = new HashSet<>();

    public static boolean startVote(String name) {
        if (status != VoteStatus.STOPPED)
            return false;

        votes = new HashSet<>();
        votes.add(name);
        status = VoteStatus.STARTED;
        Bukkit.broadcast(Config.RESTART_MESSAGE_VOTE_STARTED.replaceAll("\\{PLAYER}", name), "lagmon.restart.vote");
        voteTask = Bukkit.getScheduler().runTaskLater(LagMon.plugin, new VoteTask(), 60 * 20);
        if (checkVotes()) {
            restartVoted();
        }
        return true;
    }

    public static boolean addVote(String name) {
        if (status != VoteStatus.STARTED)
            return false;

        if (votes.contains(name))
            return false;

        votes.add(name);
        if (checkVotes()) {
            restartVoted();
        }
        return true;
    }

    public static VoteStatus getStatus() {
        return status;
    }

    private static boolean checkVotes() {
        int playerCount = Bukkit.getOnlinePlayers().size();
        int voteCount = votes.size();
        System.out.println("playerCount: " + playerCount);
        System.out.println("voteCount: " + voteCount);
        System.out.println("Percentage: " + Config.RESTART_PERCENTAGE);
        System.out.println("Needed: " + playerCount * (Config.RESTART_PERCENTAGE * 0.01));
        return voteCount >= playerCount * (Config.RESTART_PERCENTAGE * 0.01);
    }

    private static void restartVoted() {
        Bukkit.broadcastMessage(Util.color(Config.RESTART_MESSAGE_VOTE_PASSED));
        status = VoteStatus.ENDED;
        restart();
    }

    public static void restart() {
        if (voteTask != null)
            voteTask.cancel();

        if (status != VoteStatus.ENDED)
            status = VoteStatus.ENDED;

        Bukkit.getScheduler().runTaskAsynchronously(LagMon.plugin, new RestartTask());
    }

    public enum VoteStatus {
        STOPPED, // There is no vote currently going on.
        STARTED, // There is currently a vote going on.
        ENDED // A vote has just successfully finished.
    }

    private static class VoteTask implements Runnable {

        @Override
        public void run() {
            status = VoteStatus.STOPPED;
            Bukkit.broadcast(Config.RESTART_MESSAGE_VOTE_CANCELED, "lagmon.restart.vote");
        }
    }

    private static class RestartTask implements Runnable {

        @Override
        public void run() {
            Bukkit.broadcastMessage(Util.color(Config.RESART_MESSAGE_RESTART_IN.replaceAll("\\{TIME}", "10")));
            try {
                Thread.sleep(5000);
                Bukkit.broadcastMessage(Util.color(Config.RESART_MESSAGE_RESTART_IN.replaceAll("\\{TIME}", "5")));
                Thread.sleep(1000);
                Bukkit.broadcastMessage(Util.color(Config.RESART_MESSAGE_RESTART_IN.replaceAll("\\{TIME}", "4")));
                Thread.sleep(1000);
                Bukkit.broadcastMessage(Util.color(Config.RESART_MESSAGE_RESTART_IN.replaceAll("\\{TIME}", "3")));
                Thread.sleep(1000);
                Bukkit.broadcastMessage(Util.color(Config.RESART_MESSAGE_RESTART_IN.replaceAll("\\{TIME}", "2")));
                Thread.sleep(1000);
                Bukkit.broadcastMessage(Util.color(Config.RESART_MESSAGE_RESTART_IN.replaceAll("\\{TIME}", "1")));
                Thread.sleep(1000);
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
            Bukkit.getServer().shutdown();
        }
    }
}
