package nl.dragontamerfred.lagmon.components.memory;

import com.google.gson.GsonBuilder;
import nl.dragontamerfred.lagmon.Config;
import nl.dragontamerfred.lagmon.LagMon;
import nl.dragontamerfred.lagmon.Util;
import org.bukkit.Bukkit;

import javax.net.ssl.HttpsURLConnection;
import java.io.InputStreamReader;
import java.net.URL;

public class ProviderPterodactyl implements IMemoryProvider {

    private Response response = null;

    public ProviderPterodactyl() {
        Bukkit.getScheduler().runTaskTimerAsynchronously(LagMon.plugin, new task(), 0, Config.MEMORY_PTERODACTYL_POLLING_RATE * 20);
    }

    @Override
    public long getFreeRaw() {
        return (response.attributes.memory.limit - response.attributes.memory.current) * Util.BYTES_PER_MB;
    }

    @Override
    public long getUsedRaw() {
        return response.attributes.memory.current * Util.BYTES_PER_MB;
    }

    @Override
    public long getMaxRaw() {
        return response.attributes.memory.limit * Util.BYTES_PER_MB;
    }

    private class task implements Runnable {

        @Override
        public void run() {
            HttpsURLConnection connection = null;
            try {
                URL url = new URL(Config.MEMORY_PTERODACTYL_BASE_URL + "/api/client/servers/" + Config.MEMORY_PTERODACTYL_SERVER_ID + "/utilization");
                connection = (HttpsURLConnection) url.openConnection();

                connection.setRequestProperty("Authorization", "Bearer " + Config.MEMORY_PTERODACTYL_API_TOKEN);
                connection.setRequestMethod("GET");
                connection.setUseCaches(false);

                // TODO maybe connection.connect() is needed

                InputStreamReader isr = new InputStreamReader(connection.getInputStream());
                response = new GsonBuilder().create().fromJson(isr, Response.class);

                isr.close();
            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (connection != null)
                    connection.disconnect();
            }
        }
    }

    private static class Response {
        public String object;
        public Attributes attributes;
    }

    private static class Attributes {
        public String state;
        public Memory memory;
        public Cpu cpu;
        public Disk disk;
    }

    private static class Memory {
        public int current;
        public int limit;
    }

    private static class Cpu {
        public double current;
        public double[] cores;
        public double limit;
    }

    private static class Disk {
        public int current;
        public int limit;
    }
}
